@@ -7,21 +7,23 @@
  * it under the terms of version 2 of the GNU General Public License as
  * published by the Free Software Foundation.
  *
- * $Id: ReportLayers.h 1260 2006-02-02 18:43:38Z cs $
+ * $Id$
  */
 #ifndef _ReportLayers_h_
 #define _ReportLayers_h_
 
 #define TJRL_BACKGROUND 0
-#define TJRL_BACKLINES 2
-#define TJRL_OFFTIME 3
-#define TJRL_GRIDLINES 4
-#define TJRL_GRIDLABLES 5
-#define TJRL_NOWLINE 5
+#define TJRL_BACKLINES 1
+#define TJRL_OFFTIME 2
+#define TJRL_GRIDLINES 3
+#define TJRL_GRIDLABLES 4
+#define TJRL_HIGHLIGHT 5
+#define TJRL_NOWLINE 6
 #define TJRL_TASKS 10
 #define TJRL_TASKCOMP 15
 #define TJRL_DEPARROWS 16
 #define TJRL_LOADBARS 10
+#define TJRL_LOADBARFRAMES 11
 
 #endif
 
