@@ -7,7 +7,7 @@
  * it under the terms of version 2 of the GNU General Public License as
  * published by the Free Software Foundation.
  *
- * $Id: ReportManager.cpp 1214 2005-11-17 20:05:36Z cs $
+ * $Id$
  */
 
 #include "ReportManager.h"
@@ -15,7 +15,6 @@
 #include <assert.h>
 
 #include <qwidgetstack.h>
-#include <qstring.h>
 #include <qpopupmenu.h>
 
 #include <kmainwindow.h>
@@ -31,31 +30,44 @@
 #include <kmessagebox.h>
 
 #include "Project.h"
-#include "Report.h"
 #include "HTMLReport.h"
 #include "ICalReport.h"
 #include "CSVReport.h"
 #include "XMLReport.h"
 #include "ExportReport.h"
 
-#include "ManagedReportInfo.h"
-#include "TjReportBase.h"
 #include "TjSummaryReport.h"
 #include "TjHTMLReport.h"
 #include "TjTaskReport.h"
 #include "TjResourceReport.h"
 #include "MainWidget.h"
 
+// Older versions of KDE do not have this macro
+#ifndef KDE_IS_VERSION
+#define KDE_IS_VERSION(a,b,c) 0
+#endif
+
 ReportManager::ReportManager(KMainWindow* m, QWidgetStack* v,
                              KListView* b, KListViewSearchLine* s) :
     mainWindow(m), reportStack(v), browser(b), searchLine(s)
 {
-    loadingProject = FALSE;
+    loadingProject = false;
 
-    // Hide the 2nd column. It contains the report ID that is of no interest
-    // to the user.
+    /* Hide the 2nd column. It contains the report ID that is of no interest
+     * to the user. */
     browser->setColumnWidthMode(1, QListView::Manual);
     browser->hideColumn(1);
+
+    // Setup zoom toolbar actions.
+    zoomSelector = new KSelectAction
+        (i18n("Zoom"), "viewmag", 0, this, SLOT(zoomTo()),
+         m->actionCollection(), "zoom_to");
+    zoomSelector->setEditable(false);
+    zoomSelector->setComboWidth(170);
+#if KDE_IS_VERSION(3,4,89)
+    zoomSelector->setMaxComboViewCount(15);
+#endif
+
 }
 
 ReportManager::~ReportManager()
@@ -211,6 +223,11 @@ ReportManager::updateReportBrowser()
             subTypeIcon =
                 KGlobal::iconLoader()->loadIcon("tj_calendar_report",
                                                 KIcon::Small);
+        else if (strncmp(subType, "MonthlyCalendar",
+                         strlen("MonthlyCalendar")) == 0)
+            subTypeIcon =
+                KGlobal::iconLoader()->loadIcon("tj_calendar_report",
+                                                KIcon::Small);
         else if (strncmp(subType, "Status", strlen("Status")) == 0)
             subTypeIcon =
                 KGlobal::iconLoader()->loadIcon("tj_status_report",
@@ -260,7 +277,7 @@ ReportManager::generateReport(QListViewItem* lvi)
             mr = *mri;
 
     if (!mr)
-        return TRUE;
+        return true;
 
     bool retVal = true;
     if (strncmp(mr->getProjectReport()->getType(), "Qt", 2) == 0)
@@ -302,7 +319,7 @@ ReportManager::generateReport(QListViewItem* lvi)
     {
         kdDebug() << "Report type " << mr->getProjectReport()->getType()
             << " not yet supported" << endl;
-        return FALSE;
+        return false;
     }
 
     if (retVal)
@@ -343,28 +360,32 @@ ReportManager::showReport(QListViewItem* lvi, bool& showReportTab)
          * summary report. The summary report has no report definition, so it
          * can be identified by a 0 pointer. */
         for (std::list<ManagedReportInfo*>::const_iterator
-             mri = reports.begin();
-             mri != reports.end(); ++mri)
+             mri = reports.begin(); mri != reports.end(); ++mri)
             if ((*mri)->getProjectReport() == 0)
+            {
                 mr = *mri;
+                break;
+            }
     }
 
-    TjReportBase* tjr = mr->getReport();
+    TjUIReportBase* tjr = mr->getReport();
+    bool result = true;
     if (tjr == 0)
     {
         if (mr->getProjectReport() == 0)
-            tjr = new TjSummaryReport(reportStack, project);
+            tjr = new TjSummaryReport(reportStack, this, project);
         else if (strcmp(mr->getProjectReport()->getType(), "QtTaskReport") == 0)
-            tjr = new TjTaskReport(reportStack, mr->getProjectReport());
+            tjr = new TjTaskReport(reportStack, this, mr->getProjectReport());
         else if (strcmp(mr->getProjectReport()->getType(),
                         "QtResourceReport") == 0)
-            tjr = new TjResourceReport(reportStack, mr->getProjectReport());
+            tjr = new TjResourceReport(reportStack, this,
+                                       mr->getProjectReport());
         else if (strncmp(mr->getProjectReport()->getType(), "CSV", 3) == 0)
         {
             CSVReport* csvReport =
                 dynamic_cast<CSVReport*>(mr->getProjectReport());
             if (!csvReport->generate())
-                return FALSE;
+                result = false;
             // show the CSV file in preferred CSV handler
             KURL reportUrl =
                 KURL::fromPathOrURL(mr->getProjectReport()->
@@ -374,26 +395,27 @@ ReportManager::showReport(QListViewItem* lvi, bool& showReportTab)
             changeStatusBar(i18n("Displaying CSV report: '%1'")
                             .arg(mr->getProjectReport()->getFileName()));
             KRun::runURL(reportUrl, "text/x-csv");
-            return TRUE;
         }
         else if (strncmp(mr->getProjectReport()->getType(), "HTML", 4) == 0)
-            tjr = new TjHTMLReport(reportStack, mr->getProjectReport());
+            tjr = new TjHTMLReport(reportStack, this, mr->getProjectReport());
         else if (strncmp(mr->getProjectReport()->getType(), "ICal", 4) == 0)
         {
             ICalReport* icalReport =
                 dynamic_cast<ICalReport*>(mr->getProjectReport());
             if (!icalReport->generate())
-                return FALSE;
-            // show the TODO list in Korganizer
-            KURL reportUrl =
-                KURL::fromPathOrURL(mr->getProjectReport()->
-                                    getDefinitionFile());
-            reportUrl.setFileName(mr->getProjectReport()->getFileName());
+                result = false;
+            else
+            {
+                // show the TODO list in Korganizer
+                KURL reportUrl =
+                    KURL::fromPathOrURL(mr->getProjectReport()->
+                                        getDefinitionFile());
+                reportUrl.setFileName(mr->getProjectReport()->getFileName());
 
-            changeStatusBar(i18n("Displaying iCalendar: '%1'")
-                            .arg(mr->getProjectReport()->getFileName()));
-            KRun::runURL(reportUrl, "text/calendar");
-            return TRUE;
+                changeStatusBar(i18n("Displaying iCalendar: '%1'")
+                                .arg(mr->getProjectReport()->getFileName()));
+                KRun::runURL(reportUrl, "text/calendar");
+            }
         }
         else if (strncmp(mr->getProjectReport()->getType(), "Export", 6) == 0)
         {
@@ -401,52 +423,83 @@ ReportManager::showReport(QListViewItem* lvi, bool& showReportTab)
             ExportReport* exportReport =
                 dynamic_cast<ExportReport*>(mr->getProjectReport());
             if (!exportReport->generate())
-                return false;
-
-            // Get the full file name as URL and show it in the editor
-            KURL reportUrl =
-                KURL::fromPathOrURL(mr->getProjectReport()->getFullFileName());
-            if (reportUrl.url().right(4) == ".tjp")
-            {
-                changeStatusBar(i18n("Starting new TaskJuggler for '%1'")
-                    .arg(mr->getProjectReport()->getFileName()));
-                KRun::runURL(reportUrl, "application/x-tjp");
-            }
+                result = false;
             else
             {
-                emit signalEditFile(reportUrl);
-                showReportTab = false;
+                // Get the full file name as URL and show it in the editor
+                KURL reportUrl =
+                    KURL::fromPathOrURL(mr->getProjectReport()->
+                                        getFullFileName());
+                if (reportUrl.url().right(4) == ".tjp")
+                {
+                    changeStatusBar(i18n("Starting new TaskJuggler for '%1'")
+                                    .arg(mr->getProjectReport()->
+                                         getFileName()));
+                    KRun::runURL(reportUrl, "application/x-tjp");
+                }
+                else
+                {
+                    emit signalEditFile(reportUrl);
+                    showReportTab = false;
+                }
             }
-            return true;
+        }
+        else if (strncmp(mr->getProjectReport()->getType(), "XMLReport", 9)
+                 == 0)
+        {
+            bool result = mr->getProjectReport()->generate();
+            if (result)
+            {
+                KMessageBox::information
+                    (mainWindow, i18n("The report '%1' has been generated.")
+                     .arg(mr->getProjectReport()->getFileName()),
+                     QString::null, "XMLReportGeneratedInfo");
+                changeStatusBar(i18n("The report '%1' has been generated")
+                    .arg(mr->getProjectReport()->getFileName()));
+            }
+            else
+                changeStatusBar(i18n("Could not generated report '%1'")
+                    .arg(mr->getProjectReport()->getFileName()));
         }
         else
         {
             kdDebug() << "Report type " << mr->getProjectReport()->getType()
                 << " not yet supported" << endl;
-            return FALSE;
+            result = false;
         }
 
-        if (tjr)
+        if (!tjr)
         {
-            connect(tjr, SIGNAL(signalChangeStatusBar(const QString&)),
-                    this, SLOT(changeStatusBar(const QString&)));
-            connect(tjr, SIGNAL(signalEditCoreAttributes(CoreAttributes*)),
-                    this, SLOT(editCoreAttributes(CoreAttributes*)));
-
-            if (!tjr->generateReport())
-            {
-                delete tjr;
-                return FALSE;
-            }
-
-            reportStack->addWidget(tjr);
-            mr->setReport(tjr);
+            /* A report with no widget that can be embedded in the report view
+             * has been selected. We fall back to show the summary report and
+             * unselect the selected browser item. */
+            if (browser->currentItem())
+                browser->setSelected(browser->currentItem(), false);
+            tjr = new TjSummaryReport(reportStack, this, project);
         }
-    }
-    if (tjr)
-        reportStack->raiseWidget(tjr);
 
-    return TRUE;
+        connect(tjr, SIGNAL(signalChangeStatusBar(const QString&)),
+                this, SLOT(changeStatusBar(const QString&)));
+        connect(tjr, SIGNAL(signalEditCoreAttributes(CoreAttributes*)),
+                this, SLOT(editCoreAttributes(CoreAttributes*)));
+
+        if (!tjr->generateReport())
+        {
+            delete tjr;
+            if (browser->currentItem())
+                browser->setSelected(browser->currentItem(), false);
+            return false;
+        }
+
+        reportStack->addWidget(tjr);
+        mr->setReport(tjr);
+    }
+
+    // in older versions of KDE, this produces a (seemingly harmless) warning
+    //    ASSERT: "id < (int)d->m_list.count()" in ./kdeui/kactionclasses.cpp (433)
+    reportStack->raiseWidget(tjr);
+
+    return result;
 }
 
 void
@@ -470,11 +523,11 @@ ReportManager::showRMBMenu(QListViewItem* lvi, const QPoint& pos, int,
 
     // The XML reports cannot be be viewed, so we disable the entry.
     if (strncmp(mr->getProjectReport()->getType(), "XML", 3) == 0)
-        menu.setItemEnabled(1, FALSE);
+        menu.setItemEnabled(1, false);
 
     // The interactive reports can not be generated, so we disable the entry.
     if (strncmp(mr->getProjectReport()->getType(), "Qt", 2) == 0)
-        menu.setItemEnabled(2, FALSE);
+        menu.setItemEnabled(2, false);
 
     switch (menu.exec(pos))
     {
@@ -486,11 +539,11 @@ ReportManager::showRMBMenu(QListViewItem* lvi, const QPoint& pos, int,
         }
         case 2:
             errors = !generateReport(lvi);
-            showReportTab = FALSE;
+            showReportTab = false;
             break;
         case 3:
             editReport(mr->getProjectReport());
-            showReportTab = FALSE;
+            showReportTab = false;
             break;
         default:
             break;
@@ -569,6 +622,16 @@ ReportManager::print()
         getCurrentReport()->getReport()->print();
 }
 
+void
+ReportManager::zoomTo()
+{
+    if (loadingProject)
+        return;
+
+    if (getCurrentReport())
+        getCurrentReport()->getReport()->zoomTo(zoomSelector->currentText());
+}
+
 void
 ReportManager::zoomIn()
 {
@@ -589,6 +652,14 @@ ReportManager::zoomOut()
         getCurrentReport()->getReport()->zoomOut();
 }
 
+void
+ReportManager::updateZoomSelector(const QStringList& items,
+                                  unsigned int current)
+{
+    zoomSelector->setItems(items);
+    zoomSelector->setCurrentItem(current);
+}
+
 void
 ReportManager::setFocusToReport() const
 {
