@@ -7,12 +7,11 @@
  * it under the terms of version 2 of the GNU General Public License as
  * published by the Free Software Foundation.
  *
- * $Id: TjTaskReport.cpp 1368 2007-01-12 14:18:34Z cs $
+ * $Id$
  */
 
 #include "TjTaskReport.h"
 
-#include <qdict.h>
 #include <qcanvas.h>
 
 #include <klistview.h>
@@ -24,12 +23,9 @@
 #include <kmessagebox.h>
 
 #include "Project.h"
-#include "Task.h"
 #include "Resource.h"
 #include "ExpressionTree.h"
 #include "QtTaskReport.h"
-#include "QtTaskReportElement.h"
-#include "ReportLayers.h"
 #include "TjPrintTaskReport.h"
 #include "TjGanttChart.h"
 
@@ -48,7 +44,7 @@ TjTaskReport::newPrintReport(KPrinter* pr)
     return new TjPrintTaskReport(report, pr);
 }
 
-const QtReportElement*
+QtReportElement*
 TjTaskReport::getReportElement() const
 {
     return reportElement;
@@ -68,12 +64,12 @@ TjTaskReport::generateList()
     maxDepth = 0;
 
     if (!report)
-        return FALSE;
+        return false;
 
     // We need those values frequently. So let's store them in a more
     // accessible place.
-    reportElement =
-        (dynamic_cast<QtTaskReport*>(report))->getTable();
+    reportElement = dynamic_cast<QtTaskReportElement*>(
+        dynamic_cast<QtTaskReport*>(report)->getTable());
     scenario = reportElement->getScenario(0);
     taskList = report->getProject()->getTaskList();
 
@@ -87,14 +83,14 @@ TjTaskReport::generateList()
                                        reportElement->getHideTask(), et))
     {
         delete et;
-        return FALSE;
+        return false;
     }
     delete et;
 
     reportElement->sortTaskList(taskList);
 
     if (taskList.isEmpty())
-        return TRUE;
+        return true;
 
     generateListHeader(i18n("Task"), reportElement);
 
@@ -137,12 +133,12 @@ TjTaskReport::generateList()
             {
                 if (!report->isRolledUp(*tli,
                                            reportElement->getRollUpTask()))
-                    newLvi->setOpen(TRUE);
+                    newLvi->setOpen(true);
                 if (reportElement->getRollUpTask()->getErrorFlag())
-                    return FALSE;
+                    return false;
             }
             else
-                newLvi->setOpen(TRUE);
+                newLvi->setOpen(true);
         }
         else if ((*tli)->isMilestone())
         {
@@ -157,7 +153,7 @@ TjTaskReport::generateList()
                                           getBookedResourcesIterator(scenario));
                  *rli; ++rli)
             {
-                QListViewItem* lvi =
+                KListViewItem* lvi =
                     new KListViewItem(newLvi, (*rli)->getName());
                 lvi->setPixmap(0, KGlobal::iconLoader()->
                                loadIcon("tj_resource", KIcon::Small));
@@ -176,7 +172,7 @@ TjTaskReport::generateList()
     // columns.
     for (TaskListIterator tli(taskList); *tli; ++tli)
     {
-        QListViewItem* lvi = ca2lviDict[QString("t:") + (*tli)->getId()];
+        KListViewItem* lvi = ca2lviDict[QString("t:") + (*tli)->getId()];
         generateTaskListLine(reportElement, *tli, lvi);
         if (!(*tli)->isContainer() && !(*tli)->isMilestone())
         {
@@ -184,7 +180,7 @@ TjTaskReport::generateList()
                                           getBookedResourcesIterator(scenario));
                  *rli; ++rli)
             {
-                QListViewItem* lvi = ca2lviDict[QString("r:") +
+                KListViewItem* lvi = ca2lviDict[QString("r:") +
                     (*tli)->getId() + ":" + (*rli)->getId()];
                 // Make sure that the resource is not hidden.
                 if (lvi)
@@ -193,7 +189,7 @@ TjTaskReport::generateList()
         }
     }
 
-    return TRUE;
+    return true;
 }
 
 QString
@@ -226,7 +222,7 @@ TjTaskReport::generateStatusBarText(const QPoint& pos,
     {
         Resource* r = dynamic_cast<Resource*>(ca);
         Task* t = dynamic_cast<Task*>(parent);
-        double load = r->getLoad(scenario, iv, AllAccounts, t);
+        double load = r->getEffectiveLoad(scenario, iv, AllAccounts, t);
         double allocatedTimeLoad = r->getAllocatedTimeLoad
             (scenario, iv, AllAccounts, t);
         text = i18n("%1(%2) - %3:  Effort=%4  Allocated Time=%5  %6(%7)")
