@@ -8,7 +8,7 @@
  * it under the terms of version 2 of the GNU General Public License as
  * published by the Free Software Foundation.
  *
- * $Id: taskjuggler.cpp 1369 2007-01-13 20:31:07Z cs $
+ * $Id$
  */
 
 
@@ -34,7 +34,6 @@
 #include <kio/netaccess.h>
 #include <kconfig.h>
 #include <kstandarddirs.h>
-#include <kurl.h>
 #include <kurldrag.h>
 #include <kurlrequesterdlg.h>
 #include <kedittoolbar.h>
@@ -82,7 +81,7 @@ TaskJuggler::TaskJuggler()
     delayTimer = new QTimer(this);
     connect(delayTimer, SIGNAL(timeout()),
             this, SLOT(showTipOnStart()));
-    delayTimer->start(200, TRUE);
+    delayTimer->start(200, true);
 }
 
 TaskJuggler::~TaskJuggler()
@@ -159,6 +158,9 @@ void TaskJuggler::setupActions()
     new KAction(i18n("Stop scheduling"), "stop", 0,
                 m_view, SLOT(stop()),
                 actionCollection(), "stop");
+    new KAction(i18n("&Generate all Reports"), 0, 0,
+                m_view, SLOT(generate()),
+                actionCollection(), "generate");
     new KAction(i18n("Goto &previous Problem"), "tj_previous_problem",
                 KShortcut(KKey("F10")),
                 m_view, SLOT(previousProblem()),
@@ -280,7 +282,7 @@ TaskJuggler::dropEvent(QDropEvent *event)
 bool
 TaskJuggler::queryClose()
 {
-    return TRUE;
+    return true;
 }
 
 bool
@@ -288,7 +290,7 @@ TaskJuggler::queryExit()
 {
     m_view->quit(false);
     saveProperties(kapp->config());
-    return TRUE;
+    return true;
 }
 
 void
