@@ -1,35 +1,25 @@
 /*
  * Project.cpp - TaskJuggler
  *
- * Copyright (c) 2001, 2002, 2003, 2004 by Chris Schlaeger <cs@kde.org>
+ * Copyright (c) 2001, 2002, 2003, 2004, 2005, 2006
+ * by Chris Schlaeger <cs@kde.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of version 2 of the GNU General Public License as
  * published by the Free Software Foundation.
  *
- * $Id: Project.cpp 1223 2005-11-22 13:44:21Z cs $
+ * $Id$
  */
 
-#include <config.h>
-#include <stdio.h>
-#include <stdlib.h>
-#include <qdom.h>
-#include <qdict.h>
-
-#include "Interval.h"
 #include "Project.h"
-#include "debug.h"
+
+#include <stdlib.h>
+
 #include "TjMessageHandler.h"
-#include "tjlib-internal.h"
 #include "Scenario.h"
 #include "Shift.h"
 #include "Account.h"
-#include "Task.h"
 #include "Resource.h"
-#include "Utility.h"
-#include "VacationInterval.h"
-#include "Optimizer.h"
-#include "OptimizerRun.h"
 #include "HTMLTaskReport.h"
 #include "HTMLResourceReport.h"
 #include "HTMLAccountReport.h"
@@ -41,131 +31,106 @@
 #include "ExportReport.h"
 #include "ReportXML.h"
 #include "UsageLimits.h"
-#include "kotrus.h"
 #include "CustomAttributeDefinition.h"
 
 DebugController DebugCtrl;
 
-Project::Project()
+Project::Project() :
+    QObject(),
+    start(0),
+    end(0),
+    now(0),
+    allowRedefinitions(false),
+    weekStartsMonday(true),
+    name(),
+    version(),
+    copyright(),
+    customer(),
+    timeZone(),
+    timeFormat("%Y-%m-%d %H:%M"),
+    shortTimeFormat("%H:%M"),
+    currency(),
+    currencyDigits(3),
+    numberFormat("-", "", ",", ".", 1),
+    currencyFormat("(", ")", ",", ".", 0),
+    priority(500),
+    minEffort(0.0),
+    resourceLimits(0),
+    rate(0.0),
+    dailyWorkingHours(8.0),
+    yearlyWorkingDays(260.714),
+    workingHours(),
+    scheduleGranularity(ONEHOUR),
+    allowedFlags(),
+    projectIDs(),
+    currentId(),
+    maxErrors(0),
+    journal(),
+    vacationList(),
+    scenarioList(),
+    taskList(),
+    resourceList(),
+    accountList(),
+    shiftList(),
+    originalTaskList(),
+    originalResourceList(),
+    originalAccountList(),
+    taskAttributes(),
+    resourceAttributes(),
+    accountAttributes(),
+    xmlreport(0),
+    reports(),
+    interactiveReports(),
+    sourceFiles(),
+    breakFlag(false)
 {
     /* Pick some reasonable initial number since we don't know the
      * project time frame yet. */
     initUtility(20000);
 
-    vacationList.setAutoDelete(TRUE);
-    taskAttributes.setAutoDelete(TRUE);
-    resourceAttributes.setAutoDelete(TRUE);
-    reports.setAutoDelete(TRUE);
-
-    allowRedefinitions = FALSE;
-
-    currentId = QString::null;
+    vacationList.setAutoDelete(true);
+    accountAttributes.setAutoDelete(true);
+    taskAttributes.setAutoDelete(true);
+    resourceAttributes.setAutoDelete(true);
+    reports.setAutoDelete(true);
 
     new Scenario(this, "plan", "Plan", 0);
-    scenarioList.createIndex(TRUE);
-    scenarioList.createIndex(FALSE);
+    scenarioList.createIndex(true);
+    scenarioList.createIndex(false);
 
-    /* This is the version 1.0 XML reports. It should be deleted for the next
-     * major release. */
-    xmlreport = 0;
-
-    priority = 500;
-    /* The following settings are country and culture dependent. Those
-     * defaults are probably true for many Western countries, but have to be
-     * changed in project files. */
-    dailyWorkingHours = 8.0;
-    yearlyWorkingDays = 260.714;
-    scheduleGranularity = ONEHOUR;
-    weekStartsMonday = TRUE;
-    timeFormat = "%Y-%m-%d %H:%M";
-    shortTimeFormat = "%H:%M";
-    numberFormat = RealFormat("-", "", ",", ".", 1);
-    currencyFormat = RealFormat("(", ")", ",", ".", 0);
-
-    start = 0;
-    end = 0;
-    now = time(0);
-
-    minEffort = 0.0;
-    resourceLimits = 0;
-    rate = 0.0;
-    currencyDigits = 3;
-    kotrus = 0;
+    setNow(time(0));
 
     /* Initialize working hours with default values that match the Monday -
      * Friday 9 - 6 (with 1 hour lunch break) pattern used by many western
      * countries. */
     // Sunday
     workingHours[0] = new QPtrList<Interval>();
-    workingHours[0]->setAutoDelete(TRUE);
+    workingHours[0]->setAutoDelete(true);
 
     for (int i = 1; i < 6; ++i)
     {
         workingHours[i] = new QPtrList<Interval>();
-        workingHours[i]->setAutoDelete(TRUE);
+        workingHours[i]->setAutoDelete(true);
         workingHours[i]->append(new Interval(9 * ONEHOUR, 12 * ONEHOUR - 1));
         workingHours[i]->append(new Interval(13 * ONEHOUR, 18 * ONEHOUR - 1));
     }
 
     // Saturday
     workingHours[6] = new QPtrList<Interval>();
-    workingHours[6]->setAutoDelete(TRUE);
-
-    maxErrors = 0;
+    workingHours[6]->setAutoDelete(true);
 }
 
 Project::~Project()
 {
-    while (!taskList.isEmpty())
-    {
-        for (TaskListIterator tli(taskList); *tli; ++tli)
-            if ((*tli)->getParent() == 0)
-            {
-                delete *tli;
-                break;
-            }
-    }
-
-    while (!resourceList.isEmpty())
-    {
-        for (ResourceListIterator tli(resourceList); *tli; ++tli)
-            if ((*tli)->getParent() == 0)
-            {
-                delete *tli;
-                break;
-            }
-    }
+    taskList.deleteContents();
+    resourceList.deleteContents();
     Resource::deleteStaticData();
 
-    while (!accountList.isEmpty())
-    {
-        for (AccountListIterator tli(accountList); *tli; ++tli)
-            if ((*tli)->getParent() == 0)
-            {
-                delete *tli;
-                break;
-            }
-    }
+    accountList.deleteContents();
+    shiftList.deleteContents();
+    scenarioList.deleteContents();
 
-    while (!shiftList.isEmpty())
-    {
-        for (ShiftListIterator tli(shiftList); *tli; ++tli)
-            if ((*tli)->getParent() == 0)
-            {
-                delete *tli;
-                break;
-            }
-    }
-
-    while (!scenarioList.isEmpty())
-    {
-        for (ScenarioListIterator tli(scenarioList); *tli; ++tli)
-            if ((*tli)->getParent() == 0)
-            {
-                delete *tli;
-                break;
-            }
-    }
+    delete resourceLimits;
 
     // Remove support for 1.0 XML reports for next major release. */
     delete xmlreport;
@@ -200,11 +165,14 @@ Project::setProgressBar(int i, int of)
     emit updateProgressBar(i, of);
 }
 
-void
+bool
 Project::setTimeZone(const QString& tz)
 {
-    setTimezone(tz);
+    if (!setTimezone(tz))
+        return false;
+
     timeZone = tz;
+    return true;
 }
 
 Scenario*
@@ -245,18 +213,40 @@ Project::getScenarioIndex(const QString& id) const
     return scenarioList.getIndex(id);
 }
 
+void
+Project::setNow(time_t n)
+{
+    /* Align 'now' time to timing resolution. If the resolution is
+     * changed later, this has to be done again. */
+    now = (n / scheduleGranularity) * scheduleGranularity;
+}
+
+void
+Project::setWorkingHours(int day, const QPtrList<Interval>& l)
+{
+    if (day < 0 || day > 6)
+        qFatal("day out of range");
+    delete workingHours[day];
+
+    // Create a deep copy of the interval list.
+    workingHours[day] = new QPtrList<Interval>;
+    workingHours[day]->setAutoDelete(true);
+    for (QPtrListIterator<Interval> pli(l); pli; ++pli)
+        workingHours[day]->append(new Interval(**pli));
+}
+
 bool
 Project::addId(const QString& id, bool changeCurrentId)
 {
     if (projectIDs.findIndex(id) != -1)
-        return FALSE;
+        return false;
     else
         projectIDs.append(id);
 
     if (changeCurrentId)
         currentId = id;
 
-    return TRUE;
+    return true;
 }
 
 QString
@@ -282,8 +272,8 @@ Project::addScenario(Scenario* s)
 
     /* This is not too efficient, but since there are usually only a few
      * scenarios in a project, this doesn't hurt too much. */
-    scenarioList.createIndex(TRUE);
-    scenarioList.createIndex(FALSE);
+    scenarioList.createIndex(true);
+    scenarioList.createIndex(false);
 }
 
 void
@@ -316,10 +306,10 @@ bool
 Project::addTaskAttribute(const QString& id, CustomAttributeDefinition* cad)
 {
     if (taskAttributes.find(id))
-        return FALSE;
+        return false;
 
     taskAttributes.insert(id, cad);
-    return TRUE;
+    return true;
 }
 
 const CustomAttributeDefinition*
@@ -357,10 +347,10 @@ Project::addResourceAttribute(const QString& id,
                               CustomAttributeDefinition* cad)
 {
     if (resourceAttributes.find(id))
-        return FALSE;
+        return false;
 
     resourceAttributes.insert(id, cad);
-    return TRUE;
+    return true;
 }
 
 const CustomAttributeDefinition*
@@ -385,55 +375,55 @@ bool
 Project::addAccountAttribute(const QString& id,
                               CustomAttributeDefinition* cad)
 {
-    if (resourceAttributes.find(id))
-        return FALSE;
+    if (accountAttributes.find(id))
+        return false;
 
-    resourceAttributes.insert(id, cad);
-    return TRUE;
+    accountAttributes.insert(id, cad);
+    return true;
 }
 
 const CustomAttributeDefinition*
 Project::getAccountAttribute(const QString& id) const
 {
-    return resourceAttributes[id];
+    return accountAttributes[id];
 }
 
 bool
-Project::isWorkingDay(time_t d) const
+Project::isWorkingDay(time_t wd) const
 {
-    return !(workingHours[dayOfWeek(d, FALSE)]->isEmpty() ||
-             isVacation(d));
+    return !(workingHours[dayOfWeek(wd, false)]->isEmpty() ||
+             isVacation(wd));
 }
 
 bool
-Project::isWorkingTime(time_t d) const
+Project::isWorkingTime(time_t wd) const
 {
-    if (isVacation(d))
-        return FALSE;
+    if (isVacation(wd))
+        return false;
 
-    int dow = dayOfWeek(d, FALSE);
+    int dow = dayOfWeek(wd, false);
     for (QPtrListIterator<Interval> ili(*(workingHours[dow])); *ili != 0; ++ili)
     {
-        if ((*ili)->contains(secondsOfDay(d)))
-            return TRUE;
+        if ((*ili)->contains(secondsOfDay(wd)))
+            return true;
     }
-    return FALSE;
+    return false;
 }
 
 bool
 Project::isWorkingTime(const Interval& iv) const
 {
     if (isVacation(iv.getStart()))
-        return FALSE;
+        return false;
 
-    int dow = dayOfWeek(iv.getStart(), FALSE);
+    int dow = dayOfWeek(iv.getStart(), false);
     for (QPtrListIterator<Interval> ili(*(workingHours[dow])); *ili != 0; ++ili)
     {
         if ((*ili)->contains(Interval(secondsOfDay(iv.getStart()),
                                   secondsOfDay(iv.getEnd()))))
-            return TRUE;
+            return true;
     }
-    return FALSE;
+    return false;
 }
 
 int
@@ -458,36 +448,37 @@ Project::convertToDailyLoad(long secs) const
 void
 Project::addJournalEntry(JournalEntry* entry)
 {
-    journal.append(entry);
+    journal.inSort(entry);
 }
 
-JournalIterator
+Journal::Iterator
 Project::getJournalIterator() const
 {
-    return JournalIterator(journal);
+    return Journal::Iterator(journal);
 }
 
 bool
-Project::pass2(bool noDepCheck, bool& fatalError)
+Project::pass2(bool noDepCheck)
 {
+    int oldErrors = TJMH.getErrors();
+
     if (taskList.isEmpty())
     {
         TJMH.errorMessage(i18n("The project does not contain any tasks."));
-        return FALSE;
+        return false;
     }
 
     QDict<Task> idHash;
-    bool error = FALSE;
 
     /* The optimum size for the localtime hash is twice the number of time
      * slots times 2 (because of timeslot and timeslot - 1s). */
     initUtility(4 * ((end - start) / scheduleGranularity));
 
     // Generate sequence numbers for all lists.
-    taskList.createIndex(TRUE);
-    resourceList.createIndex(TRUE);
-    accountList.createIndex(TRUE);
-    shiftList.createIndex(TRUE);
+    taskList.createIndex(true);
+    resourceList.createIndex(true);
+    accountList.createIndex(true);
+    shiftList.createIndex(true);
 
     // Initialize random generator.
     srand((int) start);
@@ -499,10 +490,8 @@ Project::pass2(bool noDepCheck, bool& fatalError)
     }
     // Create cross links from dependency lists.
     for (TaskListIterator tli(taskList); *tli != 0; ++tli)
-    {
-        if (!(*tli)->xRef(idHash))
-            error = TRUE;
-    }
+        (*tli)->xRef(idHash);
+
     for (TaskListIterator tli(taskList); *tli != 0; ++tli)
     {
         // Set dates according to implicit dependencies
@@ -530,50 +519,46 @@ Project::pass2(bool noDepCheck, bool& fatalError)
 
     // Now check that all tasks have sufficient data to be scheduled.
     setProgressInfo(i18n("Checking scheduling data..."));
+    bool error = false;
     for (ScenarioListIterator sci(scenarioList); *sci; ++sci)
         for (TaskListIterator tli(taskList); *tli != 0; ++tli)
             if (!(*tli)->preScheduleOk((*sci)->getSequenceNo() - 1))
             {
-                error = TRUE;
-                fatalError = true;
+                error = true;
             }
+    if (error)
+        return false;
 
     if (!noDepCheck)
     {
         setProgressInfo(i18n("Searching for dependency loops ..."));
         if (DEBUGPS(1))
-            qDebug("Searching for dependency loops ...");
+            tjDebug("Searching for dependency loops ...");
         // Check all tasks for dependency loops.
+        LDIList chkedTaskList;
         for (TaskListIterator tli(taskList); *tli != 0; ++tli)
-            (*tli)->initLoopDetector();
-        for (TaskListIterator tli(taskList); *tli != 0; ++tli)
-            if ((*tli)->loopDetector())
-            {
-                fatalError = true;
-                return FALSE;
-            }
+            if ((*tli)->loopDetector(chkedTaskList))
+                return false;
 
         setProgressInfo(i18n("Searching for underspecified tasks ..."));
         if (DEBUGPS(1))
-            qDebug("Searching for underspecified tasks ...");
+            tjDebug("Searching for underspecified tasks ...");
         for (ScenarioListIterator sci(scenarioList); *sci; ++sci)
             for (TaskListIterator tli(taskList); *tli != 0; ++tli)
                 if (!(*tli)->checkDetermination((*sci)->getSequenceNo() - 1))
-                {
-                    error = TRUE;
-                    fatalError = true;
-                }
-        if (fatalError)
+                    error = true;
+
+        if (error)
             return false;
     }
 
-    return !error;
+    return TJMH.getErrors() == oldErrors;
 }
 
 bool
 Project::scheduleScenario(Scenario* sc)
 {
-    bool error = FALSE;
+    int oldErrors = TJMH.getErrors();
 
     setProgressInfo(i18n("Scheduling scenario %1...").arg(sc->getId()));
 
@@ -583,9 +568,8 @@ Project::scheduleScenario(Scenario* sc)
     if (!schedule(scIdx))
     {
         if (DEBUGPS(2))
-            qDebug(i18n("Scheduling errors in scenario '%1'.")
+            tjDebug(i18n("Scheduling errors in scenario '%1'.")
                    .arg(sc->getId()));
-        error = TRUE;
         if (breakFlag)
             return false;
     }
@@ -594,13 +578,10 @@ Project::scheduleScenario(Scenario* sc)
     for (ResourceListIterator rli(resourceList); *rli != 0; ++rli)
     {
         if (!(*rli)->bookingsOk(scIdx))
-        {
-            error = TRUE;
             break;
-        }
     }
 
-    return !error;
+    return TJMH.getErrors() == oldErrors;
 }
 
 void
@@ -620,16 +601,16 @@ Project::completeBuffersAndIndices()
 bool
 Project::scheduleAllScenarios()
 {
-    bool schedulingOk = TRUE;
+    bool schedulingOk = true;
     for (ScenarioListIterator sci(scenarioList); *sci; ++sci)
         if ((*sci)->getEnabled())
         {
             if (DEBUGPS(1))
-                qDebug(i18n("Scheduling scenario '%1' ...")
+                tjDebug(i18n("Scheduling scenario '%1' ...")
                        .arg((*sci)->getId()));
 
             if (!scheduleScenario(*sci))
-                schedulingOk = FALSE;
+                schedulingOk = false;
             if (breakFlag)
                 return false;
         }
@@ -645,7 +626,7 @@ Project::overlayScenario(int base, int sc)
     for (TaskListIterator tli(taskList); *tli != 0; ++tli)
         (*tli)->overlayScenario(base, sc);
 
-    for (ScenarioListIterator sli(scenarioList[base]->getSubListIterator());
+    for (ScenarioListIterator sli(scenarioList[sc]->getSubListIterator());
          *sli; ++sli)
         overlayScenario(sc, (*sli)->getSequenceNo() - 1);
 }
@@ -653,6 +634,9 @@ Project::overlayScenario(int base, int sc)
 void
 Project::prepareScenario(int sc)
 {
+    for (ResourceListIterator rli(resourceList); *rli != 0; ++rli)
+        (*rli)->prepareScenario(sc);
+
     for (TaskListIterator tli(taskList); *tli != 0; ++tli)
         (*tli)->prepareScenario(sc);
 
@@ -669,16 +653,14 @@ Project::prepareScenario(int sc)
     for (TaskListIterator tli(taskList); *tli != 0; ++tli)
         (*tli)->propagateInitialValues(sc);
 
-    for (ResourceListIterator rli(resourceList); *rli != 0; ++rli)
-        (*rli)->prepareScenario(sc);
     if (DEBUGTS(4))
     {
-        qDebug("Allocation probabilities for the resources:");
+        tjDebug("Allocation probabilities for the resources:");
         for (ResourceListIterator rli(resourceList); *rli != 0; ++rli)
             qDebug("Resource %s: %f%%",
                    (*rli)->getId().latin1(),
                    (*rli)->getAllocationProbability(sc));
-        qDebug("Criticalnesses of the tasks with respect to resource "
+        tjDebug("Criticalnesses of the tasks with respect to resource "
                "availability:");
         for (TaskListIterator tli(taskList); *tli != 0; ++tli)
             qDebug("Task %s: %-5.1f %-5.1f", (*tli)->getId().latin1(),
@@ -694,34 +676,61 @@ Project::finishScenario(int sc)
         (*rli)->finishScenario(sc);
     for (TaskListIterator tli(taskList); *tli != 0; ++tli)
         (*tli)->finishScenario(sc);
+
     /* We need to have finished the scenario for all tasks before we can
      * calculate the completion degree. */
     for (TaskListIterator tli(taskList); *tli != 0; ++tli)
         (*tli)->calcCompletionDegree(sc);
+
+    /* If the user has not set the minSlackRate to 0 we look for critical
+     * pathes. */
+    if (getScenario(sc)->getMinSlackRate() > 0.0)
+    {
+        setProgressInfo(i18n("Computing critical pathes..."));
+        /* The critical path detector needs to know the end of the last task.
+         * So we have to find this out first. */
+        time_t maxEnd = 0;
+        for (TaskListIterator tli(taskList); *tli != 0; ++tli)
+        if (maxEnd < (*tli)->getEnd(sc))
+            maxEnd = (*tli)->getEnd(sc);
+
+        for (TaskListIterator tli(taskList); *tli != 0; ++tli)
+            (*tli)->checkAndMarkCriticalPath
+                (sc, getScenario(sc)->getMinSlackRate(), maxEnd);
+    }
 }
 
 bool
 Project::schedule(int sc)
 {
-    bool error = FALSE;
+    int oldErrors = TJMH.getErrors();
 
-    TaskList fullSortedTasks(taskList);
     // The scheduling function only cares about leaf tasks. Container tasks
     // are scheduled automatically when all their childern are scheduled. So
     // we create a task list that only contains leaf tasks.
-    TaskList sortedTasks;
-    int leafTasks = 0;
-    for (TaskListIterator tli(fullSortedTasks); *tli != 0; ++tli)
+    TaskList allLeafTasks;
+    for (TaskListIterator tli(taskList); *tli != 0; ++tli)
         if (!(*tli)->hasSubs())
-        {
-            sortedTasks.append(*tli);
-            leafTasks++;
-        }
+            allLeafTasks.append(*tli);
 
-    sortedTasks.setSorting(CoreAttributesList::PrioDown, 0);
-    sortedTasks.setSorting(CoreAttributesList::PathCriticalnessDown, 1);
-    sortedTasks.setSorting(CoreAttributesList::SequenceUp, 2);
-    sortedTasks.sort();
+    allLeafTasks.setSorting(CoreAttributesList::PrioDown, 0);
+    allLeafTasks.setSorting(CoreAttributesList::PathCriticalnessDown, 1);
+    allLeafTasks.setSorting(CoreAttributesList::SequenceUp, 2);
+    allLeafTasks.sort();
+
+    /* The workItems list contains all tasks that are ready to be scheduled at
+     * any given iteration. When a tasks has been scheduled completely, this
+     * list needs to be updated again as some tasks may now have become ready
+     * to be scheduled. */
+    TaskList workItems;
+    int sortedTasks = 0;
+    for (TaskListIterator tli(allLeafTasks); *tli != 0; ++tli)
+    {
+        if ((*tli)->isReadyForScheduling())
+            workItems.append(*tli);
+        if ((*tli)->isSchedulingDone())
+            sortedTasks++;
+    }
 
     bool done;
     /* While the scheduling process progresses, the list contains more and
@@ -730,13 +739,14 @@ Project::schedule(int sc)
      * the list when we start, we initialize the timer with a very large
      * number so the first round of cleanup is done right after the first
      * scheduling pass. */
-    int cleanupTimer = 100000;
     breakFlag = false;
+    bool runAwayFound = false;
     do
     {
-        done = TRUE;
+        done = true;
         time_t slot = 0;
         int priority = 0;
+        double pathCriticalness = 0.0;
         Task::SchedulingInfo schedulingInfo = Task::ASAP;
 
         /* The task list is sorted by priority. The priority decreases towards
@@ -744,22 +754,25 @@ Project::schedule(int sc)
          * task that can be scheduled. It the determines the time slot that
          * will be scheduled during this run for all subsequent tasks as well.
          */
-        for (TaskListIterator tli(sortedTasks); *tli != 0; ++tli)
+        for (TaskListIterator tli(workItems); *tli != 0; ++tli)
         {
             if (slot == 0)
             {
                 /* No time slot has been set yet. Check if this task can be
                  * scheduled and provides a suggestion. */
                 slot = (*tli)->nextSlot(scheduleGranularity);
-                priority = (*tli)->getPriority();
-                schedulingInfo = (*tli)->getScheduling();
                 /* If not, try the next task. */
                 if (slot == 0)
                     continue;
+                priority = (*tli)->getPriority();
+                pathCriticalness = (*tli)->getPathCriticalness(sc);
+                schedulingInfo = (*tli)->getScheduling();
 
                 if (DEBUGPS(4))
-                    qDebug("Task '%s' (Prio %d) requests slot %s",
+                    qDebug("Task '%s' (Prio %d, Direction: %d) requests "
+                           "slot %s",
                            (*tli)->getId().latin1(), (*tli)->getPriority(),
+                           (*tli)->getScheduling(),
                            time2ISO(slot).latin1());
                 /* If the task wants a time slot outside of the project time
                  * frame, we flag this task as a runaway and go to the next
@@ -768,23 +781,20 @@ Project::schedule(int sc)
                     slot > (end - (time_t) scheduleGranularity + 1))
                 {
                     (*tli)->setRunaway();
-                    if (DEBUGPS(5))
-                        qDebug("Marking task '%s' as runaway",
-                               (*tli)->getId().latin1());
-                    error = TRUE;
+                    runAwayFound = true;
                     slot = 0;
                     continue;
                 }
             }
-            done = FALSE;
+            done = false;
             /* Each task has a scheduling direction (forward or backward)
              * depending on it's constrains. The task with the highest
-             * priority determins the time slot and hence the scheduling
-             * direction. Since tasks that have the other direction cannot the
-             * scheduled then, we have to stop this run as soon as we hit a
-             * task that runs in the other direction. If we would not do this,
-             * tasks with lower priority would grab resources form tasks with
-             * higher priority. */
+             * priority/pathCriticalness determins the time slot and hence the
+             * scheduling direction. Since tasks that have the other direction
+             * cannot the scheduled then, we have to stop this run as soon as
+             * we hit a task that runs in the other direction. If we would not
+             * do this, tasks with lower priority/pathCriticalness  would grab
+             * resources form tasks with higher priority. */
             if ((*tli)->getScheduling() != schedulingInfo &&
                 !(*tli)->isMilestone())
             {
@@ -796,44 +806,49 @@ Project::schedule(int sc)
             }
             /* We must avoid that lower priority tasks get resources even
              * though there are higher priority tasks that are ready to be
-             * scheduled but have a non-adjacent last slot. */
-            if ((*tli)->getPriority() < priority)
+             * scheduled but have a non-adjacent last slot. If two tasks have
+             * the same priority the pathCriticalness is being used. */
+            if ((*tli)->getPriority() < priority ||
+                ((*tli)->getPriority() == priority &&
+                 (*tli)->getPathCriticalness(sc) < pathCriticalness))
                 break;
 
             // Schedule this task for the current time slot.
-            (*tli)->schedule(sc, slot, scheduleGranularity);
+            if ((*tli)->schedule(sc, slot, scheduleGranularity))
+            {
+                workItems.clear();
+                int oldSortedTasks = sortedTasks;
+                sortedTasks = 0;
+                for (TaskListIterator tli(allLeafTasks); *tli != 0; ++tli)
+                {
+                    if ((*tli)->isReadyForScheduling())
+                        workItems.append(*tli);
+                    if ((*tli)->isSchedulingDone())
+                        sortedTasks++;
+                }
+                // Update the progress bar after every 10th completed tasks.
+                if (oldSortedTasks / 10 != sortedTasks / 10)
+                {
+                    setProgressBar(sortedTasks - allLeafTasks.count(),
+                                   sortedTasks);
+                    setProgressInfo
+                        (i18n("Scheduling scenario %1 at %2")
+                         .arg(getScenarioId(sc)).arg(time2tjp(slot)));
+                }
+            }
         }
-
-        /* Remove all fully scheduled tasks from the list in regular
-         * intervals. This is fairly expensive, so we need to balance the
-         * frequency with the performance improvements due to the shortened
-         * list. We also provide progress feedback via a signal. */
-        if (++cleanupTimer > 4000)
-        {
-            cleanupTimer = 0;
-            TaskList tmpList;
-            for (TaskListIterator tli(sortedTasks); *tli != 0; ++tli)
-                if (!(*tli)->isSchedulingDone())
-                    tmpList.append(*tli);
-            sortedTasks = tmpList;
-            setProgressBar(leafTasks - sortedTasks.count(), leafTasks);
-            setProgressInfo
-                (i18n("Scheduling scenario %1 at %1")
-                 .arg(getScenarioId(sc)).arg(time2tjp(slot)));
-        }
-
     } while (!done && !breakFlag);
 
     if (breakFlag)
     {
-        setProgressInfo(i18n(""));
+        setProgressInfo("");
         setProgressBar(0, 0);
         TJMH.errorMessage(i18n("Scheduling aborted on user request"));
         return false;
     }
 
-    if (error)
-        for (TaskListIterator tli(sortedTasks); *tli != 0; ++tli)
+    if (runAwayFound)
+        for (TaskListIterator tli(taskList); *tli != 0; ++tli)
             if ((*tli)->isRunaway())
                 if ((*tli)->getScheduling() == Task::ASAP)
                     (*tli)->errorMessage
@@ -847,17 +862,16 @@ Project::schedule(int sc)
                               "project time frame. Try using an earlier "
                               "project start date.").arg((*tli)->getId()));
 
-    if (!error)
+    if (TJMH.getErrors() == oldErrors)
         setProgressBar(100, 100);
 
     /* Check that the resulting schedule meets all the requirements that the
      * user has specified. */
     setProgressInfo(i18n("Checking schedule of scenario %1")
                     .arg(getScenarioId(sc)));
-    if (!checkSchedule(sc))
-        error = TRUE;
+    checkSchedule(sc);
 
-    return !error;
+    return TJMH.getErrors() == oldErrors;
 }
 
 void
@@ -869,23 +883,23 @@ Project::breakScheduling()
 bool
 Project::checkSchedule(int sc) const
 {
-    int errors = 0;
+    int oldErrors = TJMH.getErrors();
     for (TaskListIterator tli(taskList); *tli != 0; ++tli)
     {
         /* Only check top-level tasks, since they recursively check their sub
          * tasks. */
         if ((*tli)->getParent() == 0)
-            (*tli)->scheduleOk(sc, errors);
-        if (maxErrors > 0 && errors >= maxErrors)
+            (*tli)->scheduleOk(sc);
+        if (maxErrors > 0 && TJMH.getErrors() >= maxErrors)
         {
             TJMH.errorMessage
                 (i18n("Too many errors in %1 scenario. Giving up.")
                  .arg(getScenarioId(sc)));
-            return FALSE;
+            return false;
         }
     }
 
-    return errors == 0;
+    return TJMH.getErrors() == oldErrors;
 }
 
 Report*
@@ -903,120 +917,28 @@ Project::getReportListIterator() const
     return QPtrListIterator<Report>(reports);
 }
 
-void
+bool
 Project::generateReports() const
 {
     // Generate reports
+    int errors = 0;
     for (QPtrListIterator<Report> ri(reports); *ri != 0; ++ri)
     {
         // We generate all but Qt*Reports. Those are for the GUI version.
         if (strncmp((*ri)->getType(), "Qt", 2) != 0)
         {
             if (DEBUGPS(1))
-                qDebug(i18n("Generating report '%1' ...")
+                tjDebug(i18n("Generating report '%1' ...")
                        .arg((*ri)->getFileName()));
 
-            (*ri)->generate();
+            if (!(*ri)->generate())
+                errors++;
         }
     }
 
     generateXMLReport();
-}
 
-void
-Project::setKotrus(Kotrus* k)
-{
-    if (kotrus)
-        delete kotrus;
-    kotrus = k;
-}
-
-bool
-Project::readKotrus()
-{
-    if (!kotrus)
-        return TRUE;
-
-    for (ResourceListIterator rli(resourceList); *rli != 0; ++rli)
-        (*rli)->dbLoadBookings((*rli)->getKotrusId(), 0);
-
-    return TRUE;
-}
-
-bool
-Project::updateKotrus()
-{
-    return TRUE;
-}
-
-bool
-Project::loadFromXML( const QString& inpFile )
-{
-   QDomDocument doc;
-   QFile file( inpFile );
-
-   doc.setContent( &file );
-   qDebug(  "Loading XML " + inpFile );
-
-   QDomElement elemProject = doc.documentElement();
-
-   if( !elemProject.isNull())
-   {
-      parseDomElem( elemProject );
-   }
-   else
-   {
-      qDebug("Empty !" );
-   }
-   bool fatalError;
-   if (!pass2(TRUE, fatalError))
-       return FALSE;
-   scheduleAllScenarios();
-   return true;
-}
-
-
-void Project::parseDomElem( QDomElement& parentElem )
-{
-   QDomElement elem = parentElem.firstChild().toElement();
-
-   for( ; !elem.isNull(); elem = elem.nextSibling().toElement() )
-   {
-      QString tagName = elem.tagName();
-
-      qDebug(  "|| elemType: " + tagName );
-
-      if( tagName == "Task" )
-      {
-     QString tId = elem.attribute("Id");
-     Task *t = new Task( this, tId, QString(), 0, QString(), 0 );
-     t->inheritValues();
-
-     t->loadFromXML( elem, this  );
-      }
-      else if( tagName == "Name" )
-      {
-     setName( elem.text() );
-      }
-      else if( tagName == "Project" )
-      {
-     QString prjId = elem.attribute("Id");
-     addId( prjId );  // FIXME ! There can be more than one project id!
-
-     prjId = elem.attribute("WeekStart");
-     setWeekStartsMonday( prjId == "Mon" );
-      }
-      else if( tagName == "Version" )
-     setVersion( elem.text() );
-      else if( tagName == "Priority" )
-     setPriority( elem.text().toInt());
-      else if( tagName == "start" )
-     setStart( elem.text().toLong());
-      else if( tagName == "end" )
-     setEnd( elem.text().toLong());
-
-      // parseDomElem( elem );
-   }
+    return errors == 0;
 }
 
 bool Project::generateXMLReport() const
