@@ -7,12 +7,11 @@
  * it under the terms of version 2 of the GNU General Public License as
  * published by the Free Software Foundation.
  *
- * $Id: Scenario.h 1218 2005-11-21 13:26:55Z cs $
+ * $Id$
  */
 #ifndef _Scenario_h_
 #define _Scenario_h_
 
-#include "CoreAttributes.h"
 #include "ScenarioList.h"
 
 class Project;
@@ -20,15 +19,16 @@ class QString;
 
 class Scenario : public CoreAttributes
 {
-    friend int ScenarioList::compareItemsLevel(Scenario* r1, Scenario* r2,
+    friend int ScenarioList::compareItemsLevel(CoreAttributes* c1,
+                                               CoreAttributes* c2,
                                                int level);
 public:
-    Scenario(Project* p, const QString& i, const QString& n, Scenario* p);
+    Scenario(Project* p, const QString& i, const QString& n, Scenario* s);
     virtual ~Scenario();
 
     virtual CAType getType() const { return CA_Scenario; }
 
-    Scenario* getParent() const { return (Scenario*) parent; }
+    Scenario* getParent() const { return static_cast<Scenario*>(parent); }
 
     ScenarioListIterator getSubListIterator() const;
 
@@ -44,11 +44,19 @@ public:
     void setStrictBookings(bool s) { strictBookings = s; }
     bool getStrictBookings() const { return strictBookings; }
 
+    void setMinSlackRate(double msr) { minSlackRate = msr; }
+    double getMinSlackRate() const { return minSlackRate; }
+
+    void setMaxPaths(long l) { maxPaths = l; }
+    long getMaxPaths() const { return maxPaths; }
+
 private:
     bool enabled;
     bool projectionMode;
     bool strictBookings;
     bool optimize;
+    double minSlackRate;
+    long maxPaths;
 } ;
 
 #endif
